#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
*******************************
*                             *
*         wifiConfig          *
*                             * 
*******************************

Description
    Flask app, for an easy wifi configuration

Modules
    PyAccessPoint that uses https://pypi.org/project/hostpotd
    python-wifi modified

Developer
    Roberto Lama Rodríguez
    roberlama@gmail.com
"""


#***MODULES***
import sys,os,time,argparse
from threading import Thread
from multiprocessing import Process, Value
import urllib.request as urllib2
from flask import Flask, request, redirect, url_for, g
from PyAccessPoint import pyaccesspoint
from wifiConfig.scripts.wifi import Finder, Cell


#***GLOBAL VARS***
wlan='wlan0'
app = Flask("wifi_conf",
            static_url_path='/static')
os.environ["PYTHONOPTIMIZE"] = "1"
thr_connect = None
myWifiConf = None



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
*******************************
*                             *
*         wifiConfig          *
*                             * 
*******************************

Description
    Flask app, for an easy wifi configuration

Modules
    PyAccessPoint that uses https://pypi.org/project/hostpotd
    python-wifi modified

Developer
    Roberto Lama Rodríguez
    roberlama@gmail.com
"""


#***MODULES***
import sys,os,time,argparse
from threading import Thread
from multiprocessing import Process, Value
import urllib.request as urllib2
from flask import Flask, request, redirect, url_for, g
from PyAccessPoint import pyaccesspoint
from wifiConfig.scripts.wifi import Finder, Cell


#***GLOBAL VARS***
wlan='wlan0'
app = Flask("wifi_conf",
            static_url_path='/static')
os.environ["PYTHONOPTIMIZE"] = "1"
thr_connect = None
myWifiConf = None

class WifiConfApp():
    """
        Main app
            @description
                Uses the main class to create the access point and the flask app,
                also stop the flask app when the wifi is connected.
            @params 
                access_point
                    one dict with the following keys: wlan, inet, ip, netmask, ssid, password
                    will be passed to pyAccessPoint
                flask_app
                    another dict with keys: host and port
                    will be passed to our flask app
    """
    def __init__(self, access_point = {
                            "wlan":'wlan0', "inet":None, "ip":'192.168.0.1', "netmask":'255.255.255.0', 
                            "ssid":'MyAccessPoint', "password":'1234567890'},
                       flask_app = {
                            "host":"0.0.0.0", "port":"8080"}):
        self.connected = Value('i', 0)
        global myWifiConf
        myWifiConf = WifiConf(access_point, flask_app, self.connected)
        
    def start(self):
        main = Process(target = myWifiConf.start)
        main.start()
        while True:
            myWifiConf.reset_ap()
            self.connected.value=0
            while self.connected.value != 1:
                #print("waiting "+str(self.connected.value)) 
                time.sleep(5)
            #global thr_connect
            #print(thr_connect)
            #thr_connect.join()
            print("finished")
            print("checking network...")
            if myWifiConf.check_con():
                print("network checked")
                break
            else:
                print("ERROR: network without internet conn")
        main.terminate()
        main.join()
        
#***MAIN CLASS***
class WifiConf():
    """
        Main class
            @params 
                access_point
                    one dict with the following keys: wlan, inet, ip, netmask, ssid, password
                    will be passed to pyAccessPoint
                flask_app
                    another dict with keys: host and port
                    will be passed to our flask app
    """
    def __init__(self, access_point,flask_app,connected):
        if connected != None: 
            self.connected=connected
            self.connected.value=0
        self.host=flask_app["host"]
        self.port=flask_app["port"]
        self.wlan=access_point["wlan"]
        global wlan
        wlan=self.wlan
        self.access_point = pyaccesspoint.AccessPoint(wlan=access_point["wlan"],
                                             ip=access_point["ip"],
                                             inet=access_point["inet"],
                                             ssid=access_point["ssid"],
                                             netmask=access_point["netmask"],
                                             password=access_point["password"])
                                             
    def connect(self,req, server_name, pw):
        with app.test_request_context():
            request = req
            time.sleep(1)
            self.access_point.stop()
            os.system("sudo pyaccesspoint stop")   
            myFinder=Finder(server_name=server_name,
                            password=pw,
                            interface=self.wlan) 
            myFinder.connection()
            self.connected.value=1
            print("connected!")
        print("ok")

    def reset_ap(self):
        os.system("sudo pyaccesspoint stop")    
        self.access_point.start()
                

    def start(self):
        try:
            print("running? "+str(self.access_point.is_running()))
            app.run(host=self.host, port=self.port)
        
        except Exception as e:
            print(e)
            sys.exit()

    def check_con(self):
        try:
            urllib2.urlopen("http://www.google.com",timeout=1)
            return True
        except:
            return False

#***FLASK APP***
def after_this_request(f):
    if not hasattr(g, 'after_request_callbacks'):
        g.after_request_callbacks = []
    g.after_request_callbacks.append(f)
    return f
    
@app.after_request
def call_after_request_callbacks(response):
    for callback in getattr(g, 'after_request_callbacks', ()):
        callback(response)
    return response

@app.route("/")
def wifi_conf_page():
    html = """
           <html>
                <header>
                    <title>Wifi conf</title>
                     <link rel="stylesheet" href="/static/css/style.css">
                    <script src="/static/js/main.js"></script> 
                </header>
                <body>
                    <form action="/connect" method="post" class="login-form">
                        <h1>Wifi conf</h1>
           """ 
    for i in range(3):
        try:
            myFinder=Finder(interface=wlan)
            ans=myFinder.find()
            if ans != []:
                break
            else:
                print("empty list")
        except Exception as e:
            print(e)
    if ans != []:
        html+="""
                        <div class="form-input-material">
                            <a>SSID</a>
                            <select name="ssid">
                """
        for w in ans:
            html+="""
                                    <option value="%s">%s</option>
                  """%(w["ssid"],w["ssid"])
              
                  
        html+="""
                                </select>
                        </div>
                          <div class="form-input-material">
                            <label for="password">Password</label>
                            <input type="password" name="password" id="password" placeholder=" " autocomplete="off" class="form-control-material" required />
                          </div>
                          <button type="submit" class="btn btn-primary btn-ghost">Login</button>
                          
               """
    else:
        html+="No networks avaiable."          
    html+="""
                    </form>
              </body>
          </html>
          """
    return html


@app.route("/connect", methods=['POST'])
#@app.before_request
def connect_page():
    server_name=request.form.get("ssid")
    pw=request.form.get("password")    
    global thr_connect
    thr_connect = Thread(target=myWifiConf.connect, args=[request, server_name, pw])
    thr_connect.start()
    return "connecting..."
    
@app.route('/shutdown', methods=['POST'])
def shutdown():
    fin = request.environ.get('werkzeug.server.shutdown')
    fin()
    return 'Shutdown...'


def main():
    parser = argparse.ArgumentParser(description='A utility create a wifi hotspot on linux',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('command', choices=['start'])
    parser.add_argument('-w', "--wlan", required=False, default='wlan0',
                        help='wi-fi interface that will be used to create hotspot')
    parser.add_argument('-i', "--inet", required=False, default=None, help='forwarding interface')
    parser.add_argument('-ip', required=False, default='192.168.0.1', help='ip address of this machine in new '
                                                                            'network')
    parser.add_argument('-n', "--netmask", required=False, default='255.255.255.0',
                        help='no idea what to put here as help, if don\'t know what is it don\'t change this parameter')
    parser.add_argument('-s', "--ssid", required=False, default='MyAccessPoint', help='name of new hotspot')
    parser.add_argument('-p', "--password", required=False, default='1234567890',
                        help='password that can be used to connect to created hotspot')

    parser.add_argument('-ho', "--host", required=False, default='0.0.0.0', help='name of new hotspot')
    parser.add_argument('-po', "--port", required=False, default='8080',
                        help='password that can be used to connect to created hotspot')

    args = parser.parse_args()
    
    if args.command == 'start':
        myWifiConfApp = WifiConfApp()
        myWifiConfApp.start()

if __name__ == "__main__":
    sys.exit(main())



from setuptools import setup, find_packages
import os

if os.path.exists('README.md'):
    with open('README.md') as readme_rst_file:
        long_description = readme_rst_file.read()

else:
    long_description = 'No description'


s = setup(
    install_requires=[
        'flask',
        'PyAccessPoint'
    ],
    classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    ],
    name='wifiConfig',
    version='0.1.11',
    description='Flask app for an easy wifi configuration',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords = ['wifi', 'configuration', 'hostpot', 'flask'],
    license='MIT',
    author='Roberto Lama Rodriguez',
    author_email='roberlama@gmail.com',
    url='https://github.com/RoberWare/wifiConfig',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'wifiConfig = wifiConfig.__main__:main',
        ],
    }
)



import os
import subprocess
import re
import textwrap

class Cell(object):
    """
    Presents a Python interface to the output of iwlist.
    """

    def __init__(self):
        self.bitrates = []

    def __repr__(self):
        return 'Cell(ssid={ssid})'.format(**vars(self))

    @classmethod
    def all(cls, interface):
        """
        Returns a list of all cells extracted from the output of
        iwlist.
        """
        iwlist_scan = subprocess.check_output(['/sbin/iwlist', interface, 'scan']).decode('utf-8')

        cells = map(normalize, cells_re.split(iwlist_scan)[1:])

        return cells

    @classmethod
    def where(cls, interface, fn):
        """
        Runs a filter over the output of :meth:`all` and the returns
        a list of cells that match that filter.
        """
        return list(filter(fn, cls.all(interface)))


class Finder:
    def __init__(self, server_name=None, password=None, interface=None, country='ES'):
        self.server_name = server_name
        self.password = password
        self.interface = interface
        self.country=country
        self.main_dict = {}
        print(self.server_name,self.password)
    def find(self):
        iwlist_scan = subprocess.check_output(['/sbin/iwlist', self.interface, 'scan']).decode('utf-8')

        cells = map(normalize, cells_re.split(iwlist_scan)[1:])

        w_list=[]
        for w in list(cells):
            if w.ssid != "":
                w_list.append({"ssid":w.ssid, "signal":w.signal, "quality":w.quality, "frequency":w.frequency, 
                               "bitrates":w.bitrates, "encrypted":w.encrypted, "channel":w.channel, 
                               "address":w.address, "mode":w.mode})
        return w_list


    def run(self):
        command = """/sbin/iwlist %s scan | grep -ioE 'ssid:"(.*{}.*)'"""%(self.interface)
        result = os.popen(command.format(self.server_name))
        result = list(result)
        print(result)
        if "Device or resource busy" in result:
                return None
        else:
            ssid_list = [item.lstrip('SSID:').strip('"\n') for item in result]
            print("Successfully get ssids {}".format(str(ssid_list)))

        for name in ssid_list:
            try:
                result = self.connection(name)
            except Exception as exp:
                print("Couldn't connect to name : {}. {}".format(name, exp))
            else:
                if result:
                    print("Successfully connected to {}".format(name))

    def reset(self):
        os.system("sudo pyaccesspoint stop")
        new_f="""
            auto lo
            iface lo inet loopback

            auto %s
            #allow-hotplug %s
            iface %s inet dhcp
            wpa-conf /etc/wpa_supplicant/wpa_supplicant.conf
        """%(self.interface,self.interface,self.interface)
        with open("/etc/network/interfaces", "w+") as f:
            f.write(new_f)
            
        new_f="""
        ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev
        update_config=1
        country=%s
        """%(self.country)
        with open("/etc/wpa_supplicant/wpa_supplicant.conf", "w+") as f:
            f.write(new_f)
        #os.system("sudo ifdown wlan0")


    def connection(self):
        self.reset()
        #try:
        os.system("sudo su -c 'wpa_passphrase %s %s >> /etc/wpa_supplicant/wpa_supplicant.conf'"%(self.server_name,self.password))
        os.system("wpa_cli -i %s reconfigure"%(self.interface))
        os.system("sudo ifdown wlan0 && sudo ifup wlan0")
        #except:
        #    raise
        #else:
        #    return True
        
        
        
        #new_f="""
        #    auto lo
        #    iface lo inet loopback

        #    auto wlan0
        #    iface wlan0 inet dhcp
        #        wpa-ssid %s
        #        wpa-psk %s
        #"""%(self.server_name,self.password)
        #with open("/etc/network/interfaces", "a") as f:
        #    f.write(new_f)
        #os.system("sudo ifdown wlan0 && sudo ifup wlan0")

cells_re = re.compile(r'Cell \d+ - ')
quality_re = re.compile(r'Quality=(\d+/\d+).*Signal level=(-\d+) dBm')
frequency_re = re.compile(r'([\d\.]+ .Hz).*')


scalars = (
    'address',
    'channel',
    'mode',
)

identity = lambda x: x

key_translations = {
    'encryption key': 'encrypted',
    'essid': 'ssid',
}


def normalize_key(key):
    key = key.strip().lower()

    key = key_translations.get(key, key)

    return key.replace(' ', '')

normalize_value = {
    'ssid': lambda v: v.strip('"'),
    'frequency': lambda v: frequency_re.search(v).group(1),
    'encrypted': lambda v: v == 'on',
    'channel': int,
    'address': identity,
    'mode': identity,
}


def split_on_colon(string):
    key, _, value = map(lambda s: s.strip(), string.partition(':'))

    return key, value


def normalize(cell_block):
    """
    The cell blocks come in with the every line except the first
    indented 20 spaces.  This will remove all of that extra stuff.
    """
    lines = textwrap.dedent(' ' * 20 + cell_block).splitlines()
    cell = Cell()

    while lines:
        line = lines.pop(0)

        if line.startswith('Quality'):
            cell.quality, cell.signal = quality_re.search(line).groups()
        elif line.startswith('Bit Rates'):
            values = split_on_colon(line)[1].split('; ')

            # consume next line of bit rates, because they are split on
            # different lines, sometimes...
            while lines[0].startswith(' ' * 10):
                values += lines.pop(0).strip().split('; ')

            cell.bitrates.extend(values)
        elif ':' in line:
            key, value = split_on_colon(line)
            key = normalize_key(key)

            if key == 'ie':
                if 'Unknown' in value:
                    continue

                # consume remaining block
                values = [value]
                while lines and lines[0].startswith(' ' * 4):
                    values.append(lines.pop(0).strip())

                for word in values:
                    if 'WPA2' in word:
                        cell.encryption_type = 'wpa2-?'
                    elif 'PSK' in word:
                        cell.encryption_type = 'wpa2-psk'
                    elif '802.1x' in word:
                        cell.encryption_type = 'wpa2-eap'
            elif key in normalize_value:
                setattr(cell, key, normalize_value[key](value))
    return cell